/*
 * 
 *    VECTOR OF DOUBLES
 *    - addition
 *    - subtraction
 *    - scalar multiplication
 *    - overloaded operators
 *    - write and read from file
 *    - ...
 * 
 *    CODEBOOK VECTOR
 *    - vector with associated
 *      learning rate
 *    - easily written to /
 *      read from file
 *
 *
 */

#ifndef __VEC__
#define __VEC__

#include <iostream>
#include <fstream>
#include <strstream>
#include <iomanip>
#include <stdlib.h>
#include <math.h>
#include <string>
#include <list>
#include <queue>

#define KOMMENTAR '#'
#define BLANK ' '
#define NL '\n'
#define TAB '\t'

class vec {
 public:
  vec(int n);                              // vector of size n
  vec(const vec& v);                       // copy const.
  vec(double a[], int n);                  // init by array
  ~vec() { delete [] p; };                 // destructor
  
  int size() { return s; };
  int dim()  { return s; };
  vec add(vec v);                          // vector addition
  vec sub(vec v);                          // vector subtraction
  vec mul(double d);                       // scalar multiplication
  double dis();                            // euclidean length of vec.
  double dis(vec v);                       // euclidean distance to vec.

  double getdim(int i);                    // indexing
  void setdim(int i, double d);
  double& operator[](int i);
  vec& operator=(vec v);
 
 private:
  double* p;                               // base pointer
  int s;                                   // number of elements 
};

vec operator +(vec v, vec w);

vec operator -(vec v, vec w);

vec operator *(vec v, double d);

vec operator *(double d, vec v);

ostream& operator <<(ostream& out, vec v);

istream& operator >>(istream& in, vec& v);

vector<vec> readvecfile(char* fn);              // read a vector file

void writevecfile(vector<vec> vv, char* file);  // write a vector file

class cbvec {                              // codebookvectors    
 public:                                   // are
  vec vector;                              // a vector and an 
  double rate;                             // associated learning rate
  
  cbvec(int dim, double r) : vector(dim) {rate=r;};
  cbvec(vec v, double r) : vector(v) {rate=r;};
  cbvec(int dim) : vector(dim) {rate=1;};
  cbvec(vec v) : vector(v) {rate=1;};
};

typedef queue< cbvec, list<cbvec> > cbvecqueue;

ostream& operator <<(ostream& out, cbvec v);

istream& operator >>(istream& in, cbvec& v);

cbvecqueue readcbfile(char* fn);               // read a codebook file

void writecbfile(cbvecqueue cbq, char* file);  // write a codebook file

void newcbfile(char* file, int dim, int n);    // create a new codebook file
                                               // n vectors of dimension dim

#endif
