#include <iostream>
#include <iomanip>
#include <vector>
#include <list>
#include <queue>
#include <stdlib.h>
#include <math.h>
#include <string>

#define KOMMENTAR '#'
#define BLANK ' '
#define NL '\n'
#define TAB '\t'

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void four1(double data[], unsigned long nn, int isign)
{
	unsigned long n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;
	double tempr,tempi;

	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) {
		if (j > i) {
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m=n >> 1;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax=2;
	while (n > mmax) {
		istep=mmax << 1;
		theta=isign*(6.28318530717959/mmax);
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;
		for (m=1;m<mmax;m+=2) {
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;
				tempr=wr*data[j]-wi*data[j+1];
				tempi=wr*data[j+1]+wi*data[j];
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr;
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
}
#undef SWAP
/* (C) Copr. 1986-92 Numerical Recipes Software `521.. */
    	


int main( int argc, char* argv[] ) {

  int NN = 1024;                                 // number of values for fft 
                                                 // must be power of 2
                                                 // is also the Time Window
                                                 // Size in number of samples
                                                 // with 8 samples/msec 
                                                 // -- 128 samples = 16 ms
  int TFS = 80;                                  // Time Frame Shift in 
                                                 // number of samples
  int i;

  if(argc > 1) { 
    TFS = NN = atoi(argv[1]);
    for(i = 1; i<=NN/2; i*=2);
    NN = i;
  }
  if(argc > 2) 
    TFS = atoi(argv[2]);

  double data[2*NN];                             // vector used for fft

  string dummy;
  double samplerate = 8000;
  
  cin >> dummy >> dummy >> dummy;                // read header
  cin >> samplerate;
  
  cout << KOMMENTAR                              // write header
       << " Spectrum" << NL;        
  cout << KOMMENTAR 
       << " Sample Rate " << samplerate << NL;
  cout << KOMMENTAR
       << " Frame Size " << NN 
       << " samples = " << NN/samplerate*1000
       << " ms" << NL;
  cout << KOMMENTAR
       << " Frame Shift " << TFS
       << " samples = " << TFS/samplerate*1000
       << " ms" << NL;

  queue< double, list<double> > qin;             // input queue
  double d = 0;

  int nof = 0;                                   // number of frames
  double f;                                      // frequency
  double a;                                      // amplitude
  double phi;                                    // phase

  while(!cin.eof() && !cin.fail()) {             // as long as there is input

    while(qin.size() < (unsigned int) NN) {      // fill qin up with NN 
                                                 // elements from input
      if(!cin.eof() && !cin.fail()) {
	cin >> dummy >> d;
	qin.push(d);
      }
      else qin.push(0);
    }

    for(i=0; i<NN; i++) {                        // from qin to data vector
      data[2*i] = qin.front();
      qin.push(qin.front()), qin.pop();
      data[2*i+1] = 0;                           // phase is 0
    }

    four1(data-1, NN, 1);                        // fft data vector

    cout << NL;
    cout << KOMMENTAR 
	 << " Frame Onset " << nof*TFS 
	 << " samples = " << nof*TFS/samplerate*1000
	 << " ms" << NL;
    
    for(i=0; i<NN/2; i++) {                      // then check all frequencies
           
      f = i*samplerate/NN;                       // frequency      
      a = fabs(data[2*i]);                       // amplitude      
      phi = data[2*i+1];                         // phase

      cout << nof << TAB
	   << f   << TAB
	   << a   << TAB
	   << phi << NL;

    }

    for(i=0; i<TFS; i++) qin.pop();              // Shift Time Frame

    nof++;
    
  } // end for oft

}
