/* 
 * smooth.cpp
 * 
 * smooth the spectrum of a fouriertransformed data 
 * usage: cat data.spec | smooth > data.smooth
 *
 */

#include <stdlib.h>
#include <iostream>
#include <iomanip>
#include <string>
#include <list>
#include <queue>
#include <vector>

#define KOMMENTAR '#'
#define BLANK ' '
#define NL '\n'
#define TAB '\t'

#define READ_COMMENT(number_of_arguments)                                      {       int counter;                                                                   for(counter = 0; counter < number_of_arguments; counter ++)                      cin >> kommentar;                                                    }

#define COPY_COMMENT(number_of_arguments)                                      {       int counter;                                                                   for(counter = 0; counter < number_of_arguments; counter++)                       {                                                                                 cin >> kommentar;                                                              if(!cin.eof() && !cin.fail())                                                     cout << kommentar << BLANK;                                              }                                                                    }
 
#define COPY_COMMENT_LINE(number_of_arguments)                                         COPY_COMMENT(number_of_arguments)                                              cout << NL

#define PUSH push_back
#define POP pop_front
           
int main( int argc, char *argv[] ) {
  
  int  area = 7;
  int framesize;
  string kommentar, dummy;
  double smoothfactor[7] = {0.1, 0.4, 0.9, 1, 0.9, 0.4, 0.1};
  double totalsmooth = 3.8;
  double amplitude;
  double sum;

  /* Arguments of smooth: smooth Area SF1, SF2, ...
   *   Size: Size of Area used for smoothing, default 7
   *   SF:   Smoothing factors;
   *         Standard Values: 0.2, 0.5, 0.8, 1, 0.8, 0.5, 0.2
   * But: missing!
   */

  /**************************************  
   *                                    *
   *  Processing of Data File Header:   *
   *                                    *
   **************************************/

  cin >> dummy >> dummy >> dummy >> dummy >> dummy >> framesize;
  framesize *= 2;

  //write Header:
  cout << KOMMENTAR << " many vectors of dimension " << framesize/2 << NL; 
  cout << NL;  


  COPY_COMMENT_LINE(4);           // Vector number

  /**************************
   *                        *
   * Processing of Data:    *
   *                        *
   **************************/

  while(!cin.eof() && !cin.fail()) {
    deque< double > qamp;           // choice due to
                                    // necessity of accessing data 
                                    // WITHIN queue!!!!!
 
   
    while(qamp.size() < (unsigned int) (framesize/2))  
                                    // read one frame : size is FRAMESIZE/2
                                    // (all data occurs twice in FFT!!!!!!!) 
                                    // qamp contains amplitudes (for smoothing)
                                    // qrest contains rest data (for copying) 
      {      
	if(!cin.eof() && !cin.fail()) 
	  {
	    cin >>  amplitude ;
	    qamp.PUSH(amplitude);
	    }
      }
  

    int i;
    for(i=0; i < area/2; i++)     // copy amplitude values unchanged
                                  // until full smoothing can be done
                                  // (middle of smooth area) 
                                  // this is ok since we wanted to discard
                                  // low frequencies anyways
                                  //
                                  // change: discarded:
      {
	cout << 0 << NL;          // instead of amplitude value, 0 is outputted
                                  // result: discard 3 lowes frequencies!!!! 
       }
  
  
    while(qamp.size() >= (unsigned int) area)     // smoothing routine
      {
	sum = 0;                   // weighted sum of amplitudes  
	double smoothamp;          // smoothed amplitude
	for(i=0; i < area; i++)
	  sum += smoothfactor[i] * qamp[i];
	smoothamp = sum/totalsmooth;   
	
	cout << smoothamp << NL; //     << TAB;
	qamp.POP();                // remove first element of qamp
      } // END smoothing routine
    

    for(i=0; i < area/2; i++)                  // remove extra amplitudes 
	qamp.POP();


    while(qamp.size() > 0)                     // output remaining amplitudes
      {
	cout << qamp.front()  << NL, qamp.POP();
      }
    // Handling of one frame is complete.
    cout << NL;
    
    COPY_COMMENT_LINE(4);           // Frame number & offset

  }
  
  return 0;
   
}











